# Storybook Components

## Assumptions

- Component templates use the file extension `.twig` (instead of `.html.twig`) -
  this prevents Drupal from picking up templates and rendering them when they
  share a name with a template Drupal already knows about
- Components are structured like:

```console
[DRUPAL_ROOT]/web/themes/.../my-theme
  |- components
      |- ... organize your components as you see fit
          |- my-component-machine-name
              |- README.md
              |- thumbnail.png
              |- my-component-machine-name.twig (required)
              |- my-component-machine-name.stories.* (required)
              |- metadata.json (required)
              |- assets
                  |- static
                      (assets that won't be processed and need to be copied to dist folder)
                      |- ...
                  |- css
                      |- some-styles.scss
                      |- some-more-styles.css
                  |- js
                      |- some-javascript.js
                      |- other-javascript.js
                  |- img
                      |- img1.png
```

## Component Metadata

The component metadata JSON file is used to provide information about the
component that needs to be consistent in Drupal and in Storybook.

Extend this file if you need to provide information for a component that needs
to be available in Drupal and in your stories.

An example could be this. Imagine you have several color schemes, and you want
editors to choose from the supported schemes. Some components support setting
the color palette (like your navigation), but others do not support palettes
(like the image component). You could write a `metadata.json` like:

```json
{
  "machineName": "image",
  "name": "Image",
  "status": "READY",
  "myCustomFeaturesOrWhatever": {
    "availablePalettes": []
  }
}
```

With that you can have your Storybook integration not to show the custom palette
selector for that component. At the same time Drupal will know to hide the
dropdown for palettes when choosing the component.

At the same time since this component is not to be used directly in Drupal, but
in other components, we can flag that with
`embeddable: false`. Then our custom module can remove it from the available
components in Drupal.

Component metadata schema:

```json
{
  "$id": "https://www.drupal.org/cl_components/metadata.schema.json",
  "$schema": "http://json-schema.org/draft-07/schema#",
  "type": "object",
  "required": ["machineName", "name", "status"],
  "properties": {
    "machineName": {
      "type": "string",
      "title": "ID",
      "description": "Component ID used for reference on other Twig templates and other parts of the system",
      "examples": [
        "button",
        "module_23F"
      ]
    },
    "name": {
      "type": "string",
      "title": "Name",
      "description": "Human readable version of the machineName",
      "examples": [
        "Button",
        "Module 23F"
      ]
    },
    "componentType": {
      "type": "string",
      "title": "Component Type",
      "description": "Weather this component can be exposed to editors in Drupal (module), or they are for use in other components only (utility).",
      "enum": ["module", "utility"]
    },
    "description": {
      "type": "string",
      "title": "Description",
      "description": "Human readable description for the component",
      "examples": [ "Plain button. Only available for use in other components. Do not include directly." ]
    },
    "status": {
      "type": "string",
      "title": "Development Status",
      "description": "Use this property to identify if the component can be safely used in production.",
      "enum": ["WIP", "BETA", "READY", "DEPRECATED"]
    }
  }
}
```
