<?php

namespace Drupal\cl_components\Element;

use Drupal\cl_components\Service\ComponentRenderer;
use Drupal\Core\Render\Element\RenderElement;

/**
 * Provides a Storybook component render element.
 *
 * Properties:
 * - #component: The machine name of the component.
 * - #variant: (optional) The variant to be used for the component.
 *
 * Usage Example:
 *
 * @code
 * $build['component'] = array(
 *   '#type' => 'cl_component',
 *   '#component' => 'button',
 *   '#variant' => 'secondary',
 * );
 * @endcode
 *
 * @see \Drupal\Core\Render\Element\Textarea
 *
 * @RenderElement("cl_component")
 */
class ComponentElement extends RenderElement {

  /**
   * Expands a cl_component into an inline template with an attachment.
   *
   * @param array $element
   *   The element to process. See main class documentation for properties.
   *
   * @return array
   *   The form element.
   *
   * @throws \Drupal\cl_components\Exception\ComponentNotFoundException
   * @throws \Drupal\cl_components\Exception\TemplateNotFoundException
   * @throws \Twig\Error\LoaderError
   * @throws \Twig\Error\SyntaxError
   */
  public static function preRenderComponent(array $element): array {
    $renderer = \Drupal::service(ComponentRenderer::class);
    assert($renderer instanceof ComponentRenderer);
    $build = $renderer->createRenderArray(
      $element['#component'],
      $element['#context'] ?? [],
      $element['#variant'] ?? ''
    );
    return array_merge($element, $build);
  }

  /**
   * {@inheritdoc}
   */
  public function getInfo(): array {
    $class = static::class;
    return [
      '#pre_render' => [
        [$class, 'preRenderComponent'],
      ],
      '#component' => '',
      '#variant' => '',
      '#context' => [],
    ];
  }

}
